﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Configuration;
using System.Web.Configuration;
using Formsy.Config.Providers;
using Formsy.Config;

namespace Formsy.Providers.CustomField
{
    public class CustomFieldProviderManager
    {
        private static CustomFieldProvider m_DefaultProvider;
        private static CustomFieldProviderCollection m_Providers;

        private static string SectionGroupName = Settings.SectionName;
        private static string SectionName = "CustomFieldProviderSettings";


        static CustomFieldProviderManager()
        {
            Initialize();
        }

        private static void Initialize()
        {
            Configuration config = WebConfigurationManager.OpenWebConfiguration("~\\Web.config");
            var group = config.GetSectionGroup(SectionGroupName);

            CustomFieldProviderSettings configuration = (CustomFieldProviderSettings)group.Sections[SectionName];

            if (configuration == null)
                throw new ConfigurationErrorsException("Custom Field Provider configuration section is not set correctly.");

            m_Providers = new CustomFieldProviderCollection();

            ProvidersHelper.InstantiateProviders(configuration.Providers, m_Providers, typeof(CustomFieldProvider));

            m_Providers.SetReadOnly();

            m_DefaultProvider = m_Providers[configuration.Default];

            if (m_DefaultProvider == null)
                throw new Exception("Failed to load provider: " + configuration.Default);
        }

        public static CustomFieldProvider Provider
        {
            get
            {
                return m_DefaultProvider;
            }
        }

        public static CustomFieldProviderCollection Providers
        {
            get
            {
                return m_Providers;
            }
        }
    }
}