﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Configuration;
using System.Web.Configuration;
using Formsy.Config.Providers;
using Formsy.Config;

namespace Formsy.Providers.DataSource
{
    public class DataSourceProviderManager
    {
        private static DataSourceProvider m_DefaultProvider;
        private static DataProviderCollection m_Providers;

        private static string SectionGroupName = Settings.SectionName;
        private static string SectionName = "DataSourceProviderSettings";


        static DataSourceProviderManager()
        {
            Initialize();
        }

        private static void Initialize()
        {
            Configuration config = WebConfigurationManager.OpenWebConfiguration("~\\Web.config");
            var group = config.GetSectionGroup(SectionGroupName);

            DataSourceProviderSettings configuration = (DataSourceProviderSettings)group.Sections[SectionName];

            if (configuration == null)
                throw new ConfigurationErrorsException("SampleProvider configuration section is not set correctly.");

            m_Providers = new DataProviderCollection();

            ProvidersHelper.InstantiateProviders(configuration.Providers, m_Providers, typeof(DataSourceProvider));

            m_Providers.SetReadOnly();

            m_DefaultProvider = m_Providers[configuration.Default];

            if (m_DefaultProvider == null)
                throw new Exception("Failed to load provider: " + configuration.Default);
        }

        public static DataSourceProvider Provider
        {
            get
            {
                return m_DefaultProvider;
            }
        }

        public static DataProviderCollection Providers
        {
            get
            {
                return m_Providers;
            }
        }
    }
}