﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Configuration;
using System.Web.Configuration;
using Formsy.Config.Providers;
using Formsy.Config;

namespace Formsy.Providers.Regionalization
{
    public class RegionalizationProviderManager
    {
        private static RegionalizationProvider m_DefaultProvider;
        private static RegionalizationProviderCollection m_Providers;

        private static string SectionGroupName = Settings.SectionName;
        private static string SectionName = "RegionalizationProviderSettings";


        static RegionalizationProviderManager()
        {
            Initialize();
        }

        private static void Initialize()
        {
            Configuration config = WebConfigurationManager.OpenWebConfiguration("~\\Web.config");
            var group = config.GetSectionGroup(SectionGroupName);

            RegionalizationProviderSettings configuration = (RegionalizationProviderSettings)group.Sections[SectionName];

            if (configuration == null)
                throw new ConfigurationErrorsException("Regionalization Provider configuration section is not set correctly.");

            m_Providers = new RegionalizationProviderCollection();

            ProvidersHelper.InstantiateProviders(configuration.Providers, m_Providers, typeof(RegionalizationProvider));

            m_Providers.SetReadOnly();

            m_DefaultProvider = m_Providers[configuration.Default];

            if (m_DefaultProvider == null)
                throw new Exception("Failed to load provider: " + configuration.Default);
        }

        public static RegionalizationProvider Provider
        {
            get
            {
                return m_DefaultProvider;
            }
        }

        public static RegionalizationProviderCollection Providers
        {
            get
            {
                return m_Providers;
            }
        }
    }
}