﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Configuration;
using System.Web.Configuration;
using Formsy.Config.Providers;
using Formsy.Config;

namespace Formsy.Providers.DataStore
{
    public class DataStoreProviderManager
    {
        private static DataStoreProvider m_DefaultProvider;
        private static DataStoreProviderCollection m_Providers;

        private static string SectionGroupName = Settings.SectionName;
        private static string SectionName = "DataStoreProviderSettings";


        static DataStoreProviderManager()
        {
            Initialize();
        }

        private static void Initialize()
        {
            Configuration config = WebConfigurationManager.OpenWebConfiguration("~\\Web.config");
            var group = config.GetSectionGroup(SectionGroupName);

            DataStoreProviderSettings configuration = (DataStoreProviderSettings)group.Sections[SectionName];

            if (configuration == null)
                throw new ConfigurationErrorsException("DataStore Provider configuration section is not set correctly.");

            m_Providers = new DataStoreProviderCollection();

            ProvidersHelper.InstantiateProviders(configuration.Providers, m_Providers, typeof(DataStoreProvider));

            m_Providers.SetReadOnly();

            m_DefaultProvider = m_Providers[configuration.Default];

            if (m_DefaultProvider == null)
                throw new Exception("Failed to load provider: " + configuration.Default);
        }

        public static DataStoreProvider Provider
        {
            get
            {
                return m_DefaultProvider;
            }
        }

        public static DataStoreProviderCollection Providers
        {
            get
            {
                return m_Providers;
            }
        }
    }
}