﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Configuration;
using System.Web.Configuration;
using Formsy.Config.Providers;
using Formsy.Config;

namespace Formsy.Providers.FileSystem
{
    public class FileSystemProviderManager
    {
        private static FileSystemProvider m_DefaultProvider;
        private static FileSystemProviderCollection m_Providers;

        private static string SectionGroupName = Settings.SectionName;
        private static string SectionName = "FileSystemProviderSettings";


        static FileSystemProviderManager()
        {
            Initialize();
        }

        private static void Initialize()
        {
            Configuration config = WebConfigurationManager.OpenWebConfiguration("~\\Web.config");
            var group = config.GetSectionGroup(SectionGroupName);

            FileSystemProviderSettings configuration = (FileSystemProviderSettings)group.Sections[SectionName];

            if (configuration == null)
                throw new ConfigurationErrorsException("FileSystem Provider configuration section is not set correctly.");

            m_Providers = new FileSystemProviderCollection();

            ProvidersHelper.InstantiateProviders(configuration.Providers, m_Providers, typeof(FileSystemProvider));

            m_Providers.SetReadOnly();

            m_DefaultProvider = m_Providers[configuration.Default];

            if (m_DefaultProvider == null)
                throw new Exception("Failed to load provider: " + configuration.Default);
        }

        public static FileSystemProvider Provider
        {
            get
            {
                return m_DefaultProvider;
            }
        }

        public static FileSystemProviderCollection Providers
        {
            get
            {
                return m_Providers;
            }
        }
    }
}