﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Configuration;
using System.Web.Configuration;
using Formsy.Config.Providers;
using Formsy.Config;

namespace Formsy.Providers.Validation
{
    public class ValidationProviderManager
    {
        private static ValidationProvider m_DefaultProvider;
        private static ValidationProviderCollection m_Providers;

        private static string SectionGroupName = Settings.SectionName;
        private static string SectionName = "ValidationProviderSettings";


        static ValidationProviderManager()
        {
            Initialize();
        }

        private static void Initialize()
        {
            Configuration config = WebConfigurationManager.OpenWebConfiguration("~\\Web.config");
            var group = config.GetSectionGroup(SectionGroupName);

            ValidationProviderSettings configuration = (ValidationProviderSettings)group.Sections[SectionName];

            if (configuration == null)
                throw new ConfigurationErrorsException("Validation Provider configuration section is not set correctly.");

            m_Providers = new ValidationProviderCollection();

            ProvidersHelper.InstantiateProviders(configuration.Providers, m_Providers, typeof(ValidationProvider));

            m_Providers.SetReadOnly();

            m_DefaultProvider = m_Providers[configuration.Default];

            if (m_DefaultProvider == null)
                throw new Exception("Failed to load provider: " + configuration.Default);
        }

        public static ValidationProvider Provider
        {
            get
            {
                return m_DefaultProvider;
            }
        }

        public static ValidationProviderCollection Providers
        {
            get
            {
                return m_Providers;
            }
        }
    }
}